function [Ishift]=shiftcalc31(a,cv)

% shiftcalc31, by J.M. Daz, March 2009
%
% requires negtozero
%
% shifts the signal of one or more components in a full data matrix 
% according to an unconstrained pattern (delta E values) or to a sigmoidal change with pH 
% given by parameters in the column vector 'a' (a11,a12,a13,a21,a22,a23,...)
% where akj denotes parameter i of component j
% cv is the matrix obtained as [npH,isshift;pH,c;E,es;E,deltaE0;0,expin,expfin],where npH is
% the number of pH values, isshift is a vector in the form [1,0,2,3,...] 
% denoting with '0' the static components and with '1' (unconstrained), '2'(linear) or '3/4' (sigmoidal) 
% the components to be moved, E is potential,es is the initial estimate of unit voltammograms 
% and expin,expfin are the initial and final experiments where the component exists.
% In the case of unconstrained movement, the deltaE0 values are applied
%
% function [Ishift]=shiftcalc31(a,cv);

 
[nr,nc]=size(cv);
isshift=cv(1,2:nc);npH=cv(1,1);
ncomp=nc-1;
iunc=isempty(find(isshift==1));%existence of unconstrained shifts
if iunc==0,
 npot=(nr-npH-3)/2;
else npot=nr-npH-3;
end

pH=cv(2:npH+1,1); c=cv(2:npH+1,2:nc); 
if iunc==0,deltaE0=cv(npH+2:2*npH+1,2:nc);end
E=cv(2*npH+2:nr-2,1); v=cv(2*npH+2:nr-2,2:nc); 
expin=cv(nr-1,2:nc); expfin=cv(nr,2:nc);

%unfolding 'a' variables and applying deltaE0 values for unconstrained
%evolution

for k=1:ncomp,
	[nra,nothing]=size(a);
	if isshift(1,k)>=3;%sigmoidal evolution
		deltaE(:,k)=a(1,1)./(1+exp(a(2,1).*(pH-a(3,1))));
        if nra>4, a=a(4:nra,1); end 
	elseif isshift(1,k)==2;%sigmoidal evolution
		deltaE(:,k)=a(1,1)+a(2,1).*pH;
        if nra>3, a=a(3:nra,1); end 
	elseif isshift(1,k)==1,% unconstrained evolution
		deltaE(:,k)=deltaE0(:,k);
	elseif isshift(1,k)==0, % immobile component
		deltaE(:,k)=zeros(npH,1);  
	end	
end

for i=1:npH,
	vshift=v;%vshift is different for every experiment
	for k=1:ncomp,
		if deltaE(i,k)~=0,
			Eshift=E-deltaE(i,k);
		 	vshift(:,k)=spline(E,v(:,k),Eshift);
		end
	end
	Ishift(i,:)=negtozero(c(i,:)*vshift');%negtozero removes negatives values 
	%which appear at excessive extrapolation
end



