function [Ishift]=shiftcalc32(deltaEunfold,cv)

% shiftcalc32, by J.M. Daz, March 2009
%
% requires negtozero
%
% shifts the signal of one or more components in a full data matrix 
% according to an unconstrained pattern (delta E values) or to a sigmoidal change with pH 
% given by 'a' parameters previously fitted (a11,a12,a13,a21,a22,a23,...)
% where akj denotes parameter i of component j
% cv is the matrix obtained as [npH,isshift;pH,c;E,es;0,expin;0,expfin;zeros(3,:),afold],where npH is
% the number of pH values and afold contains the coefficients of the components moving in a 
% sigmoidal way 
% isshift is a vector in the form [1,0,2,...] 
% denoting with '0' the static components and with '1' (unconstrained) or '2/3' (sigmoidal) 
% the components to be moved, E is potential,es is the initial estimate of unit voltammograms 
% and expin,expfin are the initial and final experiments where the component exists.
% In the case of unconstrained movement, deltaEunfold contains the
% individual shifts in all experiments grouped in a column vector.
%
% function [Ishift]=shiftcal32(deltaEunfold,cv);



[nr,nc]=size(cv);
npH=cv(1,1);
npot=nr-npH-6;
ncomp=nc-1;


isshift=cv(1,2:nc); pH=cv(2:npH+1,1); c=cv(2:npH+1,2:nc); E=cv(npH+2:nr-5,1); v=cv(npH+2:nr-5,2:nc); 
expin=cv(nr-4,2:nc); expfin=cv(nr-3,2:nc);afold=cv(nr-2:nr,2:nc);

%unfolding delta E and 'a' variables

for k=1:ncomp,
	if isshift(1,k)>=3,%sigmoidal evolution
		deltaE(:,k)=afold(1,k)./(1+exp(afold(2,k).*(pH-afold(3,k))));
	elseif isshift(1,k)==2,%linear evolution
		deltaE(:,k)=afold(1,k)+afold(2,k).*pH;
  	elseif isshift(1,k)==1,% unconstrained evolution
		[nrE,nothing]=size(deltaEunfold);
		deltaE(:,k)=deltaEunfold(1:npH,1);
      if nrE>npH+1,deltaEunfold=deltaEunfold(npH+1:nrE,1); end
	else % immobile component
		deltaE(:,k)=zeros(npH,1);  
	end	
end

for i=1:npH,
	vshift=v;%vshift is different for every experiment
	for k=1:ncomp,
		if deltaE(i,k)~=0,
			Eshift=E-deltaE(i,k);
		 	vshift(:,k)=spline(E,v(:,k),Eshift);
		end
	end
	Ishift(i,:)=negtozero(c(i,:)*vshift');%removes negatives values which appear at excessive extrapolation
end



